@@ -1,5 +1,10 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
+
+# Copyright 2012 splinter authors. All rights reserved.
+# Use of this source code is governed by a BSD-style
+# license that can be found in the LICENSE file.
+
+from __future__ import with_statement
 import logging
 import subprocess
 import time
@@ -50,6 +55,12 @@ class BaseWebDriver(DriverAPI):
         # cleaning up the house
         subprocess.Popen = self.old_popen
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.quit()
+
     @property
     def title(self):
         return self.driver.title
@@ -183,10 +194,10 @@ class BaseWebDriver(DriverAPI):
         return self.find_by_xpath('//a[contains(@href, "%s")]' % partial_href, original_find="link by partial href", original_query=partial_href)
 
     def find_link_by_partial_text(self, partial_text):
-        return ElementList([self.element_class(element, self) for element in self.driver.find_elements_by_partial_link_text(partial_text)], find_by="partial text", query=partial_text)
+        return self.find_by_xpath('//a[contains(text(), "%s")]' % partial_text, original_find="link by partial text", original_query=partial_text)
 
     def find_link_by_text(self, text):
-        return ElementList([self.element_class(element, self) for element in self.driver.find_elements_by_link_text(text)], find_by="link by text", query=text)
+        return self.find_by_xpath('//a[text()="%s"]' % text, original_find="link by text", original_query=text)
 
     def find_by(self, finder, selector, original_find=None, original_query=None):
         elements = None
@@ -239,7 +250,7 @@ class BaseWebDriver(DriverAPI):
         for name, value in field_values.items():
             elements = self.find_by_name(name)
             element = elements.first
-            if element['type'] == 'text':
+            if element['type'] == 'text' or element.tag_name == 'textarea':
                 element.value = value
             elif element['type'] == 'checkbox':
                 if value:
@@ -267,16 +278,13 @@ class BaseWebDriver(DriverAPI):
                 field.click()
 
     def check(self, name):
-        field = self.find_by_name(name).first
-        field.check()
+        self.find_by_name(name).first.check()
 
     def uncheck(self, name):
-        field = self.find_by_name(name).first
-        field.uncheck()
+        self.find_by_name(name).first.uncheck()
 
     def select(self, name, value):
-        element = self.find_by_xpath('//select[@name="%s"]/option[@value="%s"]' % (name, value)).first._element
-        element.click()
+        self.find_by_xpath('//select[@name="%s"]/option[@value="%s"]' % (name, value)).first._element.click()
 
     def quit(self):
         self.driver.quit()
@@ -307,14 +315,10 @@ class WebDriverElement(ElementAPI):
         self.action_chains = ActionChains(parent.driver)
 
     def _get_value(self):
-        value = self["value"]
-        if value:
-            return value
-        else:
-            return self._element.text
+        return self["value"] or self._element.text
 
     def _set_value(self, value):
-        if  self._element.get_attribute('type') != 'file':
+        if self._element.get_attribute('type') != 'file':
             self._element.clear()
         self._element.send_keys(value)
 
@@ -324,6 +328,10 @@ class WebDriverElement(ElementAPI):
     def text(self):
         return self._element.text
 
+    @property
+    def tag_name(self):
+        return self._element.tag_name
+
     def fill(self, value):
         self.value = value
 
